import listEditor from 'wiki-list-editor';
import axios from 'axios';
import qs from 'querystring'
import $ from 'jquery';
import search from './search';
import createPageSearch from './fuse';
import Mustache from 'mustache';
import 'jquery-contextmenu';
import getCaretCoordinates from './caret-position'
import moment from 'moment'
import './styles.scss';
import '../node_modules/jquery-contextmenu/dist/jquery.contextMenu.css';

moment.locale('nl')

function addSaver(editor, saveUrl, page, beforeSave) {
    return {
        save() {
            return editor.save().then(outputData => {
                beforeSave()
                let data = {
                    'json': 1,
                    'p': page,
                    'summary': "",
                    'content': JSON.stringify(outputData),
                };
                return axios.post(saveUrl, qs.encode(data))
            })
        }
    }
}

function Indicator(element, timeout) {
    let timeoutId;

    return {
        done() {
            timeoutId = setTimeout(() => {
                element.classList.add('hidden')
            }, timeout * 1000);
        },

        setText(text) {
            if (timeoutId) {
                clearTimeout(timeoutId);
                timeoutId = null;
            }
            element.innerText = text;
            element.classList.remove('hidden')
        },
    }
}

function addIndicator(editor, indicator) {
    return {
        save() {
            editor.save().then(() => {
                indicator.setText('saved!');
                indicator.done();
            })
        }
    }
}

let holder = document.getElementById('editor');

function showSearchResults(searchTool, query, input, value, resultType) {
    let results = searchTool(query)

    if (results.length === 0) {
        return
    }

    let pos = getCaretCoordinates(input, value.selectionEnd, {})
    let off = $(input).offset()
    pos.top += off.top + pos.height
    pos.left += off.left

    const $lc = $('#link-complete')
    $lc.data('result-type', resultType)
    $lc.offset(pos)

    var template = document.getElementById('link-template').innerHTML;
    var rendered = Mustache.render(template, {results: results}, {}, ['[[', ']]']);
    $lc.html(rendered).fadeIn()
}

if (holder) {
    let editor = listEditor(holder, JSON.parse(holder.dataset.input));

    editor.on('change', function () {
        let element = document.getElementById('editor');
        let indicator = Indicator(document.getElementById('save-indicator'), 2);
        let saveUrl = element.dataset.saveurl;
        let page = element.dataset.page;

        indicator.setText('has changes...');
        addIndicator(
            addSaver(editor, saveUrl, page, () => indicator.setText('saving...')),
            indicator
        ).save()
    })

    $(document).on('keydown', '#link-complete', function (event) {
        if (!$('#link-complete:visible')) {
            return true;
        }
        const $popup = $('#link-complete')
        if (event.key === 'Escape') {
            $popup.fadeOut()
            return false
        }

        if (event.key === 'Enter') {
            const linkName = $popup.find('li.selected').text()
            $popup.trigger('popup:selected', [linkName, $popup.data('result-type')])
            $popup.fadeOut()
            return false
        }

        if (event.key === 'ArrowUp') {
            const selected = $popup.find('li.selected')
            const prev = selected.prev('li')
            if (prev.length) {
                prev.addClass('selected')
                selected.removeClass('selected')
                prev[0].scrollIntoView()
            } else {
                // move back from dropdown to input
                $popup.trigger('popup:leave')
                selected.removeClass('selected')
            }
            return false
        }

        if (event.key === 'ArrowDown') {
            const selected = $popup.find('li.selected')
            const next = selected.next('li');
            if (next.length) {
                next.addClass('selected')
                selected.removeClass('selected')
                next[0].scrollIntoView()
            }
            return false
        }

        return true
    })

    createPageSearch().then(function ({titleSearch, commandSearch, commands}) {
        editor.on('start-editing', function (input) {
            const $lc = $('#link-complete')

            $lc.on('popup:selected', function (event, linkName, resultType) {
                let value = input.value
                let end = input.selectionEnd
                if (resultType === 'link') {
                    let start = value.lastIndexOf("[[", end)
                    end += 2
                    let startAndLink = value.substring(0, start) + "[[" + linkName + "]]"
                    input.value = startAndLink + value.substring(end)
                    input.selectionStart = startAndLink.length
                    input.selectionEnd = startAndLink.length
                    input.focus()
                } else if (resultType === 'command') {
                    let start = value.lastIndexOf("/", end)
                    let commandResult = ""
                    let prefix = false
                    let adjustment = 0

                    let now = moment()

                    if (linkName === "Current Time") {
                        commandResult = now.format('HH:mm')
                    } else if (linkName === "Today") {
                        commandResult = "[[" + now.format('D MMMM YYYY') + "]]"
                    } else if (linkName === "Tomorrow") {
                        commandResult = "[[" + now.add(1, 'day').format('D MMMM YYYY') + "]]"
                    } else if (linkName === "Yesterday") {
                        commandResult = "[[" + now.add(-1, 'day').format('D MMMM YYYY') + "]]"
                    } else if (linkName === "TODO") {
                        commandResult = "#[[TODO]] "
                        prefix = true
                    } else if (linkName === "Page Reference") {
                        commandResult = "[[]]"
                        adjustment = -2
                    } else if (linkName === "Code Block") {
                        commandResult = "```\n\n```"
                        adjustment = -5
                    }

                    let startAndLink = prefix
                        ? commandResult + value.substring(0, start)
                        : value.substring(0, start) + commandResult

                    input.value = startAndLink + value.substring(end)

                    input.selectionStart = startAndLink.length + adjustment
                    input.selectionEnd = startAndLink.length + adjustment

                    input.focus()
                }
                return true
            })

            $lc.on('popup:leave', function (event) {
                input.focus()
            })

            $(input).on('keydown', function (event) {
                const isVisible = $('#link-complete:visible').length > 0;

                const $lc = $('#link-complete')
                if (event.key === 'Escape' && isVisible) {
                    $lc.fadeOut()
                    return false
                }

                if (event.key === 'ArrowDown' && isVisible) {
                    $lc.focus()
                    $lc.find('li:first-child').addClass('selected')
                    return false
                }
                return true
            })

            $(input).on('keyup', function () {
                let value = input.value
                let end = input.selectionEnd

                let insideLink = true
                let insideSearch = false

                let start = value.lastIndexOf("[[", end)
                if (start < 0) {
                    insideLink = false
                }

                let linkEnd = value.lastIndexOf("]]", end - 1)
                if (insideLink && start < linkEnd) {
                    insideLink = false
                }

                if (!insideLink) {
                    start = value.lastIndexOf("/", end)
                    insideSearch = start >= 0
                }

                if (insideSearch) {
                    let query = value.substring(start + 1, end);
                    showSearchResults(query => commandSearch.search(query), query, input, value, 'command');
                    return true
                } else if (insideLink) {
                    let query = value.substring(start + 2, end);
                    showSearchResults(query => titleSearch.search(query), query, input, value, 'link');
                    return true
                }
            })
        })

        editor.on('stop-editing', function (input) {
            $('#link-complete').off()
        })
    })

    $.contextMenu({
        selector: '.marker',
        items: {
            copy: {
                name: 'Copy',
                callback: function (key, opt) {
                    editor.copy(this).then(result => {
                        console.log(result)
                    })
                }
            }
        }
    });
}

let timeout = null;
let searchInput = document.getElementById('search-input');
search(searchInput).then(searcher => {
    searchInput.addEventListener('keyup', function (e) {
        clearTimeout(timeout);

        if (e.key === 'Escape') {
            $(searchInput).val('');
            $('#autocomplete').hide();
            return;
        }

        timeout = setTimeout(function () {
            let query = $(searchInput).val()
            if (query === '') {
                let autocomplete = document.getElementById('autocomplete');
                $(autocomplete).hide()
                return
            }
            $('#autocomplete').show()
            let result = searcher.search(query)
            const newpage = query.replace(/\s+/g, '_')
            var template = document.getElementById('result-template').innerHTML;
            var rendered = Mustache.render(template, {page: newpage, results: result}, {}, ['[[', ']]']);
            let autocomplete = document.getElementById('autocomplete');
            autocomplete.innerHTML = rendered;
        }, 500)
    })
})
